########################################################################
# Introscope EPAgent Plugin Script                               
#                                                                      
# Wily Technology (tm) Introscope(R) Version 4 Release 4.1
# Copyright (c) 1998 - 2003 Wily Technology, Inc. All Rights Reserved.
# Introscope(R) is a registered trademark of Wily Technology, Inc.
#
########################################################################
# diskStats.pl for HPUX
# ---------------------
# This Plugin Script returns metrics containing information
# about the system harddisks.
#
# diskStats.pl takes disk names as command line arguments.
# If no disk names are specified, the information about all
# disks is reported.
#
# Example: perl diskStats.pl
#            -- This reports metrics on all local disks
# Example: perl diskStats.pl / /usr
#            -- This reports metrics on only the disks '/' and '/usr'
#
########################################################################

#import our modules
use FindBin;
use lib ("$FindBin::Bin", "$FindBin::Bin/../lib/perl");
use Wily::PrintMetric;

use strict;

# Change this to change how many are reported in the main resource
my $TOP_N = 10;

# get the mounted disks specified on the command line
my $mountedDisksRegEx = '.'; # default is match all
if (scalar(@ARGV) > 0) {
    $mountedDisksRegEx = join('|', @ARGV); 
}

# Build the iostat command with the proper arguments
my $iostatCommand = 'iostat 1 2';

# Get the disk stats
my @iostatResults = `$iostatCommand`;
#### print "### iostat lines=$#iostatResults\n";
# START output on HPUX:
#line 0#
#line 1#  device    bps     sps    msps
#line 2#
#line 3# c1t15d0      0     0.0     1.0 # this line reported data since startup, once for each disk
#line 4#
#line 5# c1t15d0      3     1.0     1.0 # once for each disk
#line 6# 
# END output
# During testing, the output sometimes never gave the second interval, and was not 100% dependable.
if ( int( $#iostatResults / 2 ) != $#iostatResults / 2 ) { # check the size is even 
	@iostatResults = ();
}
elsif ( $#iostatResults >= 6 ) { # check for size of results is at least 7 rows (6 indices)
	# Get rid of the header lines
	@iostatResults = @iostatResults[3..$#iostatResults];
	# and the first reporting interval
	@iostatResults = @iostatResults[(($#iostatResults + 1)/ 2)..($#iostatResults-1)];
}
else {
	@iostatResults = ();
}
my $numDevices = 0;
my %bytesPerSec;
my $allBytesPerSec;
# parse the iostat results and report the relevant data using metrics
foreach my $line (@iostatResults) {
  # if there's a trailing \n char, get rid of it
  chomp $line;
  my @deviceStats = split(" ", $line);
  my $device = $deviceStats[0];
  next if ! defined $device || $device eq '';
  $numDevices++;
  # report the iostats
  $bytesPerSec{ $device } = int ($deviceStats[1] * 1024);
  $allBytesPerSec += $bytesPerSec{ $device };
  Wily::PrintMetric::printMetric( type        => 'LongCounter',
                                  resource    => 'Device',
                                  subresource => $device,
                                  name        => 'Bytes Transferred/sec',
                                  value       => $bytesPerSec{ $device },
                                );
  Wily::PrintMetric::printMetric( type        => 'IntCounter',
                                  resource    => 'Device',
                                  subresource => $device,
                                  name        => 'Seeks/sec',
                                  value       => int ($deviceStats[2]),
                                );
  Wily::PrintMetric::printMetric( type        => 'IntCounter',
                                  resource    => 'Device',
                                  subresource => $device,
                                  name        => 'Average seek time (ms)',
                                  value       => int ($deviceStats[3]),
                                );
}
if ( $numDevices > 0 ) {
  $allBytesPerSec = $allBytesPerSec / $numDevices;
  Wily::PrintMetric::printMetric( type        => 'LongCounter',
                                  resource    => 'Device',
                                  name        => 'Bytes Transferred/sec',
                                  value       => int ($allBytesPerSec),
                                );
  my @keys = sort { $bytesPerSec{$a} <=> $bytesPerSec{$b} || length($a) <=> length($b) || $a cmp $b } keys %bytesPerSec;
  my $topN;
  my $maxIdx = $TOP_N - 1;
  if ( $#keys < ($TOP_N -1 ) ) {
	$maxIdx = $#keys;
  }
  for my $key ( @keys[0..$maxIdx] ) {
	$topN = $topN . $key . "=" . $bytesPerSec{ $key } . ",";
  }
  Wily::PrintMetric::printMetric( type        => 'StringEvent',
                                  resource    => 'Device',
                                  name        => 'Top ' . $TOP_N . ' Least Active',
                                  value       => $topN,
                                );
}


# Disk command on HPUX
my $dfCommand = 'df -P';
# Get the disk stats
my @dfResults = `$dfCommand`;
# Output on HPUX:
#Filesystem           512-blocks  Used  Available Capacity Mounted on
#/dev/vg00/lvol4       2828848     5184  2823664     1%   /tmp
#/dev/vg00/lvol6       11329584  1576704  9752880    14%   /opt
#/dev/vg00/lvol5       11384720  7288560  4096160    65%   /home
#/dev/vg00/lvol7       3352080  2323904  1028176    70%   /usr
#/dev/vg00/lvol8       3641184  2045008  1596176    57%   /var
#/dev/vg00/lvol1       538064    57312   480752    11%   /stand
#/dev/vg00/lvol3       407728   160416   247312    40%   /
# Get rid of the header lines for each command
@dfResults = @dfResults[1..$#dfResults];
my $allFreeDiskPercent;
my %freeDiskPercent;
my $numDisks = 0;
foreach my $dfLine (@dfResults) {
  # get rid of any trailing \n char
  chomp $dfLine;
  my @dfStats = split(" ", $dfLine);
  my $diskName = $dfStats[5];
  # now, check to see if the user specified this disk on the command
  # line.
  next if $diskName !~ /$mountedDisksRegEx/i;
  next if ! defined $diskName || $diskName eq '';
  $numDisks++;
  # report the df stats
  Wily::PrintMetric::printMetric( type        => 'StringEvent',
                                  resource    => 'Disk',
                                  subresource => $diskName,
                                  name        => 'Filesystem',
                                  value       => $dfStats[0],
                                );
  # Just print the Free Disk Space as a Percent
  # chop gets rid of '%' in the capacity
  chop $dfStats[4];
  $freeDiskPercent{ $diskName } = int ( 100 - $dfStats[4] );
  $allFreeDiskPercent += $freeDiskPercent{ $diskName } ;
  ### print "### allFreeDiskPercent=$allFreeDiskPercent\n";
  Wily::PrintMetric::printMetric( type        => 'IntCounter',
                                  resource    => 'Disk',
                                  subresource => $diskName,
                                  name        => 'Free Disk Space (%)',
                                  value       => $freeDiskPercent{ $diskName },
                                );
  Wily::PrintMetric::printMetric( type        => 'LongCounter',
                                  resource    => 'Disk',
                                  subresource => $diskName,
                                  name        => 'Free Disk Space (bytes)',
                                  value       => int ($dfStats[3] * 512),
                                );
  Wily::PrintMetric::printMetric( type        => 'LongCounter',
                                  resource    => 'Disk',
                                  subresource => $diskName,
                                  name        => 'Total Disk Space (bytes)',
                                  value       => int ($dfStats[1] * 512),
                               );
  Wily::PrintMetric::printMetric( type        => 'LongCounter',
                                  resource    => 'Disk',
                                  subresource => $diskName,
                                  name        => 'Used Disk Space (bytes)',
                                  value       => int ($dfStats[2] * 512),
                                );
}
if ( $numDisks > 0 ) {
  $allFreeDiskPercent = $allFreeDiskPercent / $numDisks;
  Wily::PrintMetric::printMetric( type        => 'LongCounter',
                                  resource    => 'Disk',
                                  name        => 'Free Disk Space (%)',
                                  value       => int ($allFreeDiskPercent),
                                );
  my @keys = sort { $freeDiskPercent{$a} <=> $freeDiskPercent{$b} || length($a) <=> length($b) || $a cmp $b } keys %freeDiskPercent;
  my $topN;
  my $maxIdx = $TOP_N - 1;
  if ( $#keys < ($TOP_N - 1) ) {
	$maxIdx = $#keys;
  }
  for my $key ( @keys[0..$maxIdx] ) {
	$topN = $topN . $key . "=" . $freeDiskPercent{ $key } . ",";
  }
  Wily::PrintMetric::printMetric( type        => 'StringEvent',
                                  resource    => 'Disk',
                                  name        => 'Top ' . $TOP_N . ' Least Free',
                                  value       => $topN,
                                );
}